use std;

fn compute_intcode(p: &mut Vec<usize>) {
    for idx in (0..(p.len())).step_by(4) {
        match p[idx] {
            1 => {
                let left = p[idx + 1];
                let right = p[idx + 2];
                let res = p[idx + 3];
                p[res] = p[left] + p[right];
            }
            2 => {
                let left = p[idx + 1];
                let right = p[idx + 2];
                let res = p[idx + 3];
                p[res] = p[left] * p[right];
            }
            99 => break,
            _ => panic!("invalid opcode"),
        }
    }
}

pub fn part_one() {
    let mut input: Vec<usize> = std::include_str!("../day2-input.txt")
        .trim()
        .split(",")
        .map(|x| x.parse().unwrap())
        .collect();

    input[1] = 12;
    input[2] = 2;

    compute_intcode(&mut input);

    println!("Intcode [0] is: {}", input[0]);
}

pub fn part_two() {
    let mut input: Vec<usize> = std::include_str!("../day2-input.txt")
        .trim()
        .split(",")
        .map(|x| x.parse().unwrap())
        .collect();

    input[1] = 0;
    input[2] = 0;

    let lookup_num = 19690720;
    let mut found = false;

    for i in 0..99 {
        input[1] = i;
        for j in 0..99 {
            input[2] = j;
            let mut computed = input.clone();
            compute_intcode(&mut computed);

            found = computed[0] == lookup_num;

            if found {
                let (noun, verb) = (computed[1], computed[2]);
                println!(
                    "100 * noun({}) + verb({}) = {}",
                    noun,
                    verb,
                    100 * noun + verb
                );
                break;
            }
        }
        if found {
            break;
        }
    }
}

#[test]
fn test_part_one() {
    let mut intcode = vec![1, 0, 0, 0, 99];
    compute_intcode(&mut intcode);
    assert_eq!(intcode, vec!(2, 0, 0, 0, 99));

    let mut intcode = vec![2, 3, 0, 3, 99];
    compute_intcode(&mut intcode);
    assert_eq!(intcode, vec!(2, 3, 0, 6, 99));

    let mut intcode = vec![2, 4, 4, 5, 99, 0];
    compute_intcode(&mut intcode);
    assert_eq!(intcode, vec!(2, 4, 4, 5, 99, 9801));

    let mut intcode = vec![1, 1, 1, 4, 99, 5, 6, 0, 99];
    compute_intcode(&mut intcode);
    assert_eq!(intcode, vec!(30, 1, 1, 4, 2, 5, 6, 0, 99));
}
